#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_TreeNode2c.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 11)
      mexErrMsgTxt("11 input arguments are required");
   if (nlhs > 3)
      mexErrMsgTxt("Too many outputs");


   int KPASSIGN1_IN   = 0;
   int ROWS1_IN       = 1;
   int COLS1_IN       = 2;
   int SZRNG1_IN      = 3;
   int KPASSIGN2_IN   = 4;
   int ROWS2_IN       = 5;
   int COLS2_IN       = 6;
   int SZRNG2_IN      = 7;
   int NHISTBIN_IN    = 8;
   int ALPHA_IN       = 9;
   int TOLFAC_IN      = 10;

   int nKeyPnt1 = mxGetM(prhs[KPASSIGN1_IN]);
   int rows1 = (int) mxGetScalar(prhs[ROWS1_IN]);
   int cols1 = (int) mxGetScalar(prhs[COLS1_IN]);
   int nKeyPnt2 = mxGetM(prhs[KPASSIGN2_IN]);
   int rows2 = (int) mxGetScalar(prhs[ROWS2_IN]);
   int cols2 = (int) mxGetScalar(prhs[COLS2_IN]);
   int histSize = (int) mxGetScalar(prhs[NHISTBIN_IN]);
   double alpha  = mxGetScalar(prhs[ALPHA_IN]);
   double tolFac = mxGetScalar(prhs[TOLFAC_IN]);

   bool Output = false;

   /* Get the pointers to the input Data */
   double *Im1AssignPtr = mxGetPr(prhs[KPASSIGN1_IN]);
   int **Im1Assign = buildMatrix<int>(nKeyPnt1, 3);
   for (int j=0; j<3; j++){
      for (int i=0; i< nKeyPnt1; i++){
         Im1Assign[i][j] = ((int) (*Im1AssignPtr));
         Im1AssignPtr++;
      }
   }

   double *Im2AssignPtr = mxGetPr(prhs[KPASSIGN2_IN]);
   int **Im2Assign = buildMatrix<int>(nKeyPnt2, 3);
   for (int j=0; j<3; j++){
      for (int i=0; i< nKeyPnt2; i++){
         Im2Assign[i][j] = ((int) (*Im2AssignPtr));
         Im2AssignPtr++;
      }
   }


   double *szRng1Ptr = mxGetPr(prhs[SZRNG1_IN]);
   int *szRng1 = buildVector<int>(4);
   for (int k=0; k < 4; k++){
      szRng1[k] = (int) (*szRng1Ptr);
      szRng1Ptr++;
   }
   SizeRange szRange1(szRng1[2], szRng1[3], szRng1[0], szRng1[1]);

   double *szRng2Ptr = mxGetPr(prhs[SZRNG2_IN]);
   int *szRng2 = buildVector<int>(4);
   for (int k=0; k < 4; k++){
      szRng2[k] = (int) (*szRng2Ptr);
      szRng2Ptr++;
   }
   SizeRange szRange2(szRng2[2], szRng2[3], szRng2[0], szRng2[1]);

   if (Output){
      mexPrintf("rows1: %d, cols1: %d, nKeyPnt1: %d, rows2: %d, cols2: %d, nKeyPnt2: %d, histSize: %d\n",
            rows1, cols1, nKeyPnt1, rows2, cols2, nKeyPnt2, histSize);
      mexPrintf("szRange1: %s, szRange2: %s\n", szRange1.str().c_str(), szRange2.str().c_str());
   }

   WindowPair bestPair = findBoxes2c(Im1Assign, nKeyPnt1, rows1, cols1, szRange1,
         Im2Assign, nKeyPnt2, rows2, cols2, szRange2, histSize, alpha, tolFac);

   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(4, 1, mxDOUBLE_CLASS, mxREAL);
   plhs[1] = mxCreateNumericMatrix(4, 1, mxDOUBLE_CLASS, mxREAL);

   double *labelOutPtr = mxGetPr(plhs[0]);

   labelOutPtr[0] = (double) (bestPair.win1.ul_x + 1);
   labelOutPtr[1] = (double) (bestPair.win1.ul_y + 1);
   labelOutPtr[2] = (double) (bestPair.win1.lr_x + 1);
   labelOutPtr[3] = (double) (bestPair.win1.lr_y + 1);

   labelOutPtr = mxGetPr(plhs[1]);
   labelOutPtr[0] = (double) (bestPair.win2.ul_x + 1);
   labelOutPtr[1] = (double) (bestPair.win2.ul_y + 1);
   labelOutPtr[2] = (double) (bestPair.win2.lr_x + 1);
   labelOutPtr[3] = (double) (bestPair.win2.lr_y + 1);

   //If require the energy
   if (nlhs >= 3){
      int ***integralIms1 = cmpIntegralIms1(Im1Assign, nKeyPnt1, rows1, cols1, histSize);
      int *hist1 = bestPair.win1.getHist(integralIms1, histSize);
      int ***integralIms2 = cmpIntegralIms1(Im2Assign, nKeyPnt2, rows2, cols2, histSize);
      int *hist2 = bestPair.win2.getHist(integralIms2, histSize);

      double energy = cmpEnergy5(hist1, hist2, histSize, alpha);
      plhs[2] = mxCreateDoubleScalar(energy);

      destroyArrayThree<int>(integralIms1, rows1, cols1, histSize);
      destroyArrayThree<int>(integralIms2, rows2, cols2, histSize);
      destroyVector<int>(hist1, histSize);
      destroyVector<int>(hist2, histSize);
   }

   destroyMatrix<int>(Im1Assign, nKeyPnt1, 3);
   destroyMatrix<int>(Im2Assign, nKeyPnt2, 3);
   destroyVector<int>(szRng1, 4);
   destroyVector<int>(szRng2, 4);
}
