#include "mex.h"
#include <cmath>
#include <stdlib.h>
#include <stdio.h>
#include <queue>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_Multi1DSegs.h"

using namespace std;

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 2) mexErrMsgTxt("2 input arguments are required");
   if (nlhs>1) mexErrMsgTxt("Too many outputs");

   int PIXVALS_IN = 0;
   int NSEG_IN = 1;

   int len = mxGetM(prhs[PIXVALS_IN]);
   double *pixValsPtr = mxGetPr(prhs[PIXVALS_IN]);
   double *pixVals = (double *) mxCalloc(len, sizeof(double));

   for (int i=0; i < len; i++) pixVals[i] = pixValsPtr[i];

   int nSeg = (int)mxGetScalar(prhs[NSEG_IN]);

   bool Output = 0;
   if (Output) printf("cols: %d, nSeg: %d\n", len, nSeg);

   int nChosen;
   double **chosenSegs = findMulti1DSegs2(pixVals, len, nSeg, &nChosen);

   if (Output) printf("# chosen segments: %d\n", nChosen);

   /* Create the outGoing Array  */
   plhs[0] = mxCreateDoubleMatrix(3, nChosen, mxREAL);
   double *labelOutPtr = mxGetPr(plhs[0]);

   for (int k=0; k < nChosen; k++){
      labelOutPtr[0] = (double) (chosenSegs[0][k] + 1);
      labelOutPtr[1] = (double) (chosenSegs[1][k] + 1);
      labelOutPtr[2] = (double) chosenSegs[2][k];
      labelOutPtr += 3;
   }

   destroyMatrix<double>(chosenSegs, 3, nChosen);
}
