/*
 * m_mexFindNormMulti1DSegs.cpp
 *
 * Created on: Aug 13, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#include "mex.h"
#include <cmath>
#include <stdlib.h>
#include <stdio.h>
#include <queue>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_NormMulti1DSegs.h"
#include "m_Multi1DSegs.h"

using namespace std;

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 5) mexErrMsgTxt("2 input arguments are required");
   if (nlhs>2) mexErrMsgTxt("Too many outputs");

   int PIXVALS_IN = 0;
   int NSEG_IN    = 1;
   int ALPHA_IN   = 2;
   int BETA_IN    = 3;
   int TOL_IN     = 4;

//   (double *pixVals, int length, int nSeg,
//         double alpha, double beta, double tol)

   int len = mxGetM(prhs[PIXVALS_IN]);
   double *pixValsPtr = mxGetPr(prhs[PIXVALS_IN]);
   double *pixVals = (double *) mxCalloc(len, sizeof(double));
   for (int i=0; i < len; i++) pixVals[i] = pixValsPtr[i];

   int nSeg = (int) mxGetScalar(prhs[NSEG_IN]);
   double alpha = mxGetScalar(prhs[ALPHA_IN]);
   double beta = mxGetScalar(prhs[BETA_IN]);
   double tol = mxGetScalar(prhs[TOL_IN]);

   bool Output = 0;
   if (Output) printf("length: %d, nSeg: %d, alpha: %g, beta: %g, tol: %g\n",
         len, nSeg, alpha, beta, tol);

   NormMulti1DSegs rslt = findNormMulti1DSegs(pixVals, len, nSeg, alpha, beta, tol);

   if (Output) printf("# chosen segments: %d\n", rslt.nSeg);

   /* Create the outGoing Array */
   plhs[0] = mxCreateDoubleMatrix(3, rslt.nSeg, mxREAL);
   double *labelOutPtr = mxGetPr(plhs[0]);

   for (int k=0; k < rslt.nSeg; k++){
      labelOutPtr[0] = (double) (rslt.segs[0][k] + 1);
      labelOutPtr[1] = (double) (rslt.segs[1][k] + 1);
      labelOutPtr[2] = (double) rslt.segs[2][k];
      labelOutPtr += 3;
   }

   plhs[1] = mxCreateDoubleScalar(rslt.nIter);
}
