/*
 * m_mexKmeansSeg.cpp
 *
 * Created on: Dec 10, 2010
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */
#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_multiLabelKSeg.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 5)
      mexErrMsgTxt("Five input arguments are required");
   if (nlhs > 2)
      mexErrMsgTxt("Too many outputs");

   int D_IN    = 0;
   int CS_IN   = 1;
   int LMIN_IN = 2;
   int LMAX_IN = 3;
   int SHLD_NRM_IN = 4;

   int d    = mxGetM(prhs[D_IN]);
   int n    = mxGetN(prhs[D_IN]);
   int nClass = mxGetN(prhs[CS_IN]);
   int lmin = (int) mxGetScalar(prhs[LMIN_IN]);
   int lmax = (int) mxGetScalar(prhs[LMAX_IN]);
   bool shldNrm = (bool) mxGetScalar(prhs[SHLD_NRM_IN]);

   bool Output = false;

   /* Get the pointers to the input Data */
   double *DPtr = mxGetPr(prhs[D_IN]);

   double **D = buildMatrix<double>(n, d);
   for (int j = 0; j < n; j++){
      memcpy(D[j], DPtr, d*sizeof(double));
      DPtr += d;
   }

   double **Cs = buildMatrix<double>(nClass, d);
   double *CsPtr = mxGetPr(prhs[CS_IN]);
   for (int j=0; j < nClass; j++){
      memcpy(Cs[j], CsPtr, d*sizeof(double));
      CsPtr+= d;
   }

   if (Output) mexPrintf("nClass: %d, d: %d, len: %d, lmin: %d, lmax: %d, shldNrm: %d\n",
         nClass, d, n, lmin, lmax, shldNrm);

   double energy;
   vector<MLKS_Rslt> rslt = KmeansSeg(D, d, n, Cs, nClass, lmin, lmax, shldNrm, energy);

   /* Create the outGoing Array  */
   plhs[0] = mxCreateDoubleMatrix(3, rslt.size(), mxREAL);
   double *assgn = mxGetPr(plhs[0]);

   for (int j = (int) rslt.size() -1; j >=0; j--){
      *assgn++ = rslt[j].startIdx + 1;
      *assgn++ = rslt[j].endIdx + 1;
      *assgn++ = rslt[j].label + 1;
   }

   plhs[1] = mxCreateDoubleScalar(energy);
   destroyMatrix<double>(D, n, d);
   destroyMatrix<double>(Cs, nClass, d);
}
