/*
 * m_mexMultiLabelKSeg.cpp
 *
 * Created on: Nov 30, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */
#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_multiLabelKSeg.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 2)
      mexErrMsgTxt("2 input arguments are required");
   if (nlhs > 2)
      mexErrMsgTxt("Too many outputs");

   int A_IN   = 0;
   int NSEG_IN = 1;
   int len = mxGetM(prhs[A_IN]);
   int nLabel = mxGetN(prhs[A_IN]);
   int nSeg = (int)mxGetScalar(prhs[NSEG_IN]);

   bool Output = false;


   /* Get the pointers to the input Data */
   double *APtr = mxGetPr(prhs[A_IN]);
   double **A = buildMatrix<double>(nLabel, len);
   for (int j = 0; j < nLabel; j++){
      for (int i = 0; i < len; i++) A[j][i] = (double)(*APtr++);
   }

   if (Output) mexPrintf("nLabel: %d, nSeg: %d, len: %d\n", nLabel, nSeg, len);

   double energy;
   vector<MLKS_Rslt> rslt = multiLabelKseg(A, nLabel, len, nSeg, &energy);

   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(3, rslt.size(), mxDOUBLE_CLASS, mxREAL);
   double *assgn = mxGetPr(plhs[0]);

   for (int j = (int) rslt.size() -1; j >=0; j--){
      *assgn++ = rslt[j].startIdx + 1;
      *assgn++ = rslt[j].endIdx + 1;
      *assgn++ = rslt[j].label + 1;
   }

   plhs[1] = mxCreateDoubleScalar(energy);

   destroyMatrix<double>(A, nLabel, len);
}
