/*
 * m_mexRandomSeg.cpp
 *
 * Created on: Sep 21, 2010
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */
#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_multiLabelKSeg.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 5)
      mexErrMsgTxt("Three input arguments are required");
   if (nlhs > 2)
      mexErrMsgTxt("Too many outputs");

   int LEN_IN     = 0;
   int NLABEL_IN  = 1;
   int LMIN_IN    = 2;
   int LMAX_IN    = 3;
   int SEED_IN    = 4;

   int len     = (int) mxGetScalar(prhs[LEN_IN]);
   int nLabel  = (int) mxGetScalar(prhs[NLABEL_IN]);
   int lmin    = (int) mxGetScalar(prhs[LMIN_IN]);
   int lmax    = (int) mxGetScalar(prhs[LMAX_IN]);

   int seed    = (int) mxGetScalar(prhs[SEED_IN]);
   srand(seed);


   bool Output = false;

   if (Output) mexPrintf("nLabel: %d, len: %d, lmin: %d, lmax: %d\n",
         nLabel, len, lmin, lmax);

   double nValidSeg;
   vector<MLKS_Rslt> rslt = randomSeg(len, nLabel, lmin, lmax, &nValidSeg);

   /* Create the outGoing Array  */
   plhs[0] = mxCreateDoubleMatrix(3, rslt.size(), mxREAL);
   double *assgn = mxGetPr(plhs[0]);
   for (int j = (int) rslt.size() -1; j >=0; j--){
      *assgn++ = rslt[j].startIdx + 1;
      *assgn++ = rslt[j].endIdx + 1;
      *assgn++ = rslt[j].label + 1;
   }

   plhs[1] = mxCreateDoubleScalar(nValidSeg);
}
