function [label, energy] = ml_KmeansSeg(D, Cs, lmin, lmax, shldNrm)
% function [label, energy] = ml_KmeansSeg(D, Cs, lmin, lmax)
% Divide a time series into segments that minimize Kmeans energy
% This function find GLOBAL minimum using dynamic programming
% Inputs:
%   D: d*n matrix
%   Cs: d*m matrix for k-means centroids
%   lmin, lmax: the minimum and maximum lengths of each resulting segment. 
% Output:
%   label: 3*nSeg matrix label(:,i) is [start, end, label]' of segment i^th.
% Examples:
%   D = [0.1  0.1 0.1 0.1 -0.1 -0.2;
%        0.2 -0.1 0.2 0.2  0.2  0.2];      
%   Cs = [0.1 -0.1; 0.2 0.2];
%   lmin = 2; lmax =3;
%   [label, energy] = ml_KmeansSeg(D, Cs, lmin, lmax)
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 11 Dec 10

if (lmin < 1)
    error('lmin should be at least 1');
end;

if (lmin > lmax)
    error('lmin should not be greater than lmax');
end;

[label, energy] = m_mexKmeansSeg(D, Cs, lmin, lmax, shldNrm);
if isinf(energy) 
    label = [];
end
   
function test1()
D = [0.1  0.1 0.1 0.1 -0.1 -0.2;
    0.2 -0.1 0.2 0.2  0.2  0.2];
Cs = [0.1 -0.1; 0.2 0.2];
lmin = 2; lmax =3;
shldNrm = 0;
[label, energy] = ml_KmeansSeg(D, Cs, lmin, lmax, shldNrm);

