function [compressedImAssign, compressedBinWeights] = ...
    ml_compressHist(imAssign, binWeights, tol)
% Remove histogram bins that correspond to zero weights.
% This will increase the speed significantly if the weight vector
% binWeights is sparse.
% Inputs:
%   imAssign: a imH*imW matrix for histogram bin assignments of pixels, 
%       entries must be integers from 1 to k, where k is number of
%       histogram bins
%   binWeights: a k*1 vector of weights for histogram bins, weights can be
%       negative
%   tol: tolerance to decide if something is zeros or not.
% Outputs:
%   compressedImAssign: a imH*imW matrix for histogram bin assignments of 
%       pixels, entries must be integers from 1 to m, where m is number of
%       bins of compressed histogram. All bins that have zero weights are
%       now mapped to bin 1. Other bins might be assigned different bin
%       numbers.
%   compressedBinWeights: a m*1 vector of weights for histogram bins of the
%       the compressed histogram. 
% By Minh Hoai Nguyen (minhhoai@cmu.edu).
% Date: 20 Dec 2008


 [compressedImAssign, compressedBinWeights] = ...
    m_mexCompressHist(imAssign, binWeights, tol);

