function [bestBox, E] = ml_findBox1b2(pixVals, szRange)
% function [bestBox, E] = ml_findBox1b2(pixVals)
% function [bestBox, E] = ml_findBox1b2(pixVals, szRange)
% Find a rectangle that minimizes the sum of contained elements. 
% Inputs:
%   pixVals: a imH*imW matrix of positive/negative scalars.
%	szRange: a 4*1 vector for [wmin; wmax; hmin; hmax], range for width and height
% Output:
%   bestBox: a 4*1 vector for the left, top, right, bottom of the rectangle
%   E: energy
% Examples:
%   imH = 100; imW = 200;
%   pixVals = ones(imH, imW);
%   pixVals(50:70,100:150) = -1;
%   [bbox, E] = ml_findBox1b2(pixVals) 
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 6 Aug 09

[imH, imW] = size(pixVals);

if exist('szRange','var') && ~isempty(szRange)
    if (size(szRange, 1) ~= 4) || (size(szRange, 2) ~= 1)
        error('szRange should be a 4*1 vector');
    end;

    if (szRange(1) > szRange(2)) || (szRange(3) > szRange(4)) ||...
        szRange(2) <= 0 || szRange(4) <=0 || ...
        szRange(1) > imW || szRange(3) > imH
        error('invalid size range');
    end;
    if nargout >= 2
        [bestBox, E] = m_mexFindBox1b2(pixVals, szRange);
    else
        bestBox = m_mexFindBox1b2(pixVals, szRange);
    end;
else
    if nargout >= 2
        [bestBox, E] = m_mexFindBox1b2(pixVals);
    else
        bestBox = m_mexFindBox1b2(pixVals);
    end;
end;
    




