/*
 * m_NormMulti1DSegs.h
 *
 * Created on: Aug 13, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#ifndef M_NORMMULTI1DSEGS_H_
#define M_NORMMULTI1DSEGS_H_

#include "m_Multi1DSegs.h"

class NormMulti1DSegs: public Multi1DSegs{
public:
   int nIter;
   NormMulti1DSegs():Multi1DSegs(){};
};

/**
 * Find the best normalized k-segmentation S of a 1D signal that minimizes:
 *    \f$ \frac{\sum_{i \in S}pixVals(i) + \alpha}{length(S) + \beta} \f$
 * \param pixVals a 1*length 1D signal
 * \param length the length of the 1D signal
 * \param nSeg number of allowable disjoint intervals
 * \param alpha a constant; bigger alpha leads to longer k-segmentation
 * \param beta a positive constant; the bigger beta,
 *    the less important the segmentation length
 * \param tol tolerance for optimality. The objective value is guaranteed to be
 *    within tol of the returned value.
 * \return the best normalized k-segmentation. The result is given as an instance
 *    of Multi1DSegs class. It contains the number of chosen disjoint intervals
 *    and the starts and beginning of the intervals.
 * \see findMulti1DSegs2 findMulti1DSegs_seq2
 */
NormMulti1DSegs findNormMulti1DSegs(double *pixVals, int length, int nSeg,
      double alpha, double beta, double tol);


#endif /* M_NORMMULTI1DSEGS_H_ */
