/*
 * m_TreeNode1j.cpp
 *
 * Created on: Oct 12, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#include "m_TreeNode1j.h"
#include "m_malloc.h"
#include "m_BranchBound.cpp"
#include <iostream>

using namespace std;

double ****TreeNode1j1::integralIms = NULL;
int    ***TreeNode1j1::integralIms2 = NULL;
SizeRange TreeNode1j1::szRange;
double **TreeNode1j1::B = NULL;
int TreeNode1j1::nHistBin = 0;
int TreeNode1j1::nPC = 0;
double TreeNode1j1::alpha = 0.0;

void
TreeNode1j1::cmpBnds(){
   Res xx, yy;

   if ((!findEnds(Res(winPair.win1.ul_x, winPair.win1.lr_x),
        Res(winPair.win2.ul_x, winPair.win2.lr_x),
        Res(szRange.wmin, szRange.wmax), &xx)) ||
      (!findEnds(Res(winPair.win1.ul_y, winPair.win1.lr_y),
        Res(winPair.win2.ul_y, winPair.win2.lr_y),
        Res(szRange.hmin, szRange.hmax), &yy))){

       lb = INF;
       ub = INF;
       return;
   }

   ubWin = Window(xx.x, yy.x, xx.y, yy.y);
   if ((ubWin.ul_x > ubWin.lr_x) ||
      (ubWin.ul_y > ubWin.lr_y)){
       ubWin = Window();
   }

   int *hist_ub = ubWin.getHist(integralIms2, nHistBin);
   ub = 0;
   for (int i=0; i <nHistBin; i++) ub += hist_ub[i]*hist_ub[i];
   ub = ub/(1+alpha);

   for (int i= 0; i < nPC; i++){
      double *hist2_ub = ubWin.getPosNegScore(integralIms[i]);
      double projVal = hist2_ub[0] - hist2_ub[1];
      ub -= projVal*projVal;
      destroyVector<double>(hist2_ub, 2);
   }

   Window smallestWin = winPair.getSmallestWin();
   Window largestWin  = winPair.getLargestWin();

   int *hist_min = smallestWin.getHist(integralIms2, nHistBin);
   int *hist_max = largestWin.getHist(integralIms2, nHistBin);


   double l1 = 0.0, u1 = 0.0; // lower and upper bound of ||h||^2
   for (int i=0; i < nHistBin; i++)  {
      l1 += hist_min[i]*hist_min[i];
      u1 += hist_max[i]*hist_max[i];
   }

   double u2 = 0.0; // upper bound of ||Bh||^2
   for (int i=0; i < nPC; i++){
      double *hist2_min = smallestWin.getPosNegScore(integralIms[i]);
      double *hist2_max = largestWin.getPosNegScore(integralIms[i]);

      double a1 = hist2_min[0] - hist2_max[1];
      double a2 = hist2_max[0] - hist2_min[1];

      destroyVector<double>(hist2_min, 2);
      destroyVector<double>(hist2_max, 2);
      u2 += max(a1*a1, a2*a2);
      if (u2 >= u1) break;
   }

   // the lower bound is derived based on:
   // l1 <= ||h||^2 <= u1; ||Bh||^2 <= u2; ||bh||^2 <= ||h||^2.
   if (u2 <= l1){
      lb = l1/(1+alpha) - u2;
   } else if (u2 <= u1){
      lb = -alpha/(1+alpha)*u2;
   } else {
      lb = -alpha/(1+alpha)*u1;
   }

   destroyVector<int>(hist_ub, nHistBin);
   destroyVector<int>(hist_min, nHistBin);
   destroyVector<int>(hist_max, nHistBin);
}

double ****TreeNode1j2::integralIms = NULL;
int    ***TreeNode1j2::integralIms2 = NULL;
SizeRange TreeNode1j2::szRange;
double **TreeNode1j2::B = NULL;
int TreeNode1j2::nHistBin = 0;
int TreeNode1j2::nPC = 0;
double TreeNode1j2::alpha = 0.0;


void
TreeNode1j2::cmpBnds(){
   Res xx, yy;

   if ((!findEnds(Res(winPair.win1.ul_x, winPair.win1.lr_x),
        Res(winPair.win2.ul_x, winPair.win2.lr_x),
        Res(szRange.wmin, szRange.wmax), &xx)) ||
      (!findEnds(Res(winPair.win1.ul_y, winPair.win1.lr_y),
        Res(winPair.win2.ul_y, winPair.win2.lr_y),
        Res(szRange.hmin, szRange.hmax), &yy))){

       lb = INF;
       ub = INF;
       return;
   }

   ubWin = Window(xx.x, yy.x, xx.y, yy.y);
   if ((ubWin.ul_x > ubWin.lr_x) || (ubWin.ul_y > ubWin.lr_y)){
       ubWin = Window();
   }

   int *hist_ub = ubWin.getHist(integralIms2, nHistBin);
   double ub_denom = 0;
   for (int i=0; i <nHistBin; i++) ub_denom += hist_ub[i]*hist_ub[i];
   double ub_numer = 0;
   for (int i= 0; i < nPC; i++){
      double *hist2_ub = ubWin.getPosNegScore(integralIms[i]);
      double projVal = hist2_ub[0] - hist2_ub[1];
      ub_numer -= projVal*projVal;
      destroyVector<double>(hist2_ub, 2);
   }
   ub = ub_numer/(ub_denom + alpha);

   Window smallestWin = winPair.getSmallestWin();
   Window largestWin  = winPair.getLargestWin();

   int *hist_min = smallestWin.getHist(integralIms2, nHistBin);
   int *hist_max = largestWin.getHist(integralIms2, nHistBin);

   double l1 = 0.0, u1 = 0.0; //lower and upper bounds of ||h||^2
   for (int i=0; i < nHistBin; i++)  {
      l1 += hist_min[i]*hist_min[i];
      u1 += hist_max[i]*hist_max[i];
   }

   double u2 = 0.0; // upper bound of ||Bh||^2
   for (int i=0; i < nPC; i++){
      double *hist2_min = smallestWin.getPosNegScore(integralIms[i]);
      double *hist2_max = largestWin.getPosNegScore(integralIms[i]);

      double a1 = hist2_min[0] - hist2_max[1];
      double a2 = hist2_max[0] - hist2_min[1];
      destroyVector<double>(hist2_min, 2);
      destroyVector<double>(hist2_max, 2);
      u2 += max(a1*a1, a2*a2);
      if (u2 >= u1) break;
   }

   // the lower bound is derived based on:
   // l1 <= ||h||^2 <= u1; ||Bh||^2 <= u2; ||bh||^2 <= ||h||^2.
   if (u2 <= l1){
      lb = -u2/(l1 + alpha);
   } else if (u2 <= u1){
      lb = -u2/(u2 + alpha);
   } else {
      lb = -u1/(u1 + alpha);
   }

   destroyVector<int>(hist_ub, nHistBin);
   destroyVector<int>(hist_min, nHistBin);
   destroyVector<int>(hist_max, nHistBin);
}




Window findBox1j1(int **kpAssign, int nKeyPnt, int imH, int imW,
      double **B, int nHistBin, int nPC, SizeRange szRange, double alpha, double tolFac){

   double ****integralIms = new double***[nPC];
   for (int i=0; i < nPC; i++)
      integralIms[i] = cmpIntegralIms3(kpAssign, nKeyPnt, imH, imW, B[i], nHistBin);
   int ***integralIms2 = cmpIntegralIms1(kpAssign, nKeyPnt, imH, imW, nHistBin);

   TreeNode1j1::integralIms = integralIms;
   TreeNode1j1::integralIms2 = integralIms2;
   TreeNode1j1::szRange = szRange;
   TreeNode1j1::B = B;
   TreeNode1j1::nHistBin = nHistBin;
   TreeNode1j1::nPC = nPC;
   TreeNode1j1::alpha = alpha;


   Window biggestWin(0, 0, imW - 1, imH - 1);
   TreeNode1j1 rootNode(WindowPair(biggestWin, biggestWin));
   TreeNode1j1 bestNode = m_branch_bound<TreeNode1j1>(rootNode, tolFac);

   for (int i=0; i < nPC; i++) destroyArrayThree<double>(integralIms[i], imH, imW, 2);
   delete [] integralIms;
   destroyArrayThree<int>(integralIms2, imH, imW, nHistBin);
   return bestNode.ubWin;

}

Window findBox1j2(int **kpAssign, int nKeyPnt, int imH, int imW,
      double **B, int nHistBin, int nPC, SizeRange szRange, double alpha, double tolFac){

   double ****integralIms = new double***[nPC];
   for (int i=0; i < nPC; i++)
      integralIms[i] = cmpIntegralIms3(kpAssign, nKeyPnt, imH, imW, B[i], nHistBin);
   int ***integralIms2 = cmpIntegralIms1(kpAssign, nKeyPnt, imH, imW, nHistBin);

   TreeNode1j2::integralIms = integralIms;
   TreeNode1j2::integralIms2 = integralIms2;
   TreeNode1j2::szRange = szRange;
   TreeNode1j2::B = B;
   TreeNode1j2::nHistBin = nHistBin;
   TreeNode1j2::nPC = nPC;
   TreeNode1j2::alpha = alpha;

   Window biggestWin(0, 0, imW - 1, imH - 1);
   TreeNode1j2 rootNode(WindowPair(biggestWin, biggestWin));
   TreeNode1j2 bestNode = m_branch_bound<TreeNode1j2>(rootNode, tolFac);

   for (int i=0; i < nPC; i++) destroyArrayThree<double>(integralIms[i], imH, imW, 2);
   delete [] integralIms;
   destroyArrayThree<int>(integralIms2, imH, imW, nHistBin);
   return bestNode.ubWin;

}
