/*
 * m_TreeNode1j.h
 *
 * Created on: Oct 12, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#ifndef M_TREENODE1J_H_
#define M_TREENODE1J_H_

#include "m_TreeNode1.h"

/**
 *
 */
class TreeNode1j1:public TreeNode1{
public:
   static double ****integralIms, **B, alpha;
   static int    ***integralIms2, nHistBin, nPC;
   static SizeRange szRange;

   Window ubWin;

   TreeNode1j1(WindowPair winPair_):TreeNode1(winPair_){};
   TreeNode1j1():TreeNode1(){};
   void cmpBnds();
};

/**
 *
 */
class TreeNode1j2:public TreeNode1{
public:
   static double ****integralIms, **B, alpha;
   static int    ***integralIms2, nHistBin, nPC;
   static SizeRange szRange;
   Window ubWin;

   TreeNode1j2(WindowPair winPair_):TreeNode1(winPair_){};
   TreeNode1j2():TreeNode1(){};
   void cmpBnds();
};


/**
 * Find the best rectangular window in an image that minimizes the PCA residual:
 *    \f$ \frac{1}{1+\alpha}||h||^2 - ||Bh||^2\f$
 * \param kpAssign a 2-dim array of size nKeyPnt*3 for histogram bin assignments.
 *    kpAssign[i][0], kpAssign[i][1], kpAssign[i][2] are row#, col#, bin#
 *       entries are positive integers.
 * \param nKeyPnt # of keypoints.
 * \param imH, imW height and width of the image
 * \param B nPC*nHistBin matrix for the PCA ORTHORNORMAL basis
 * \param nHistBin # of histogram bins
 * \param nPC # of principal components
 * \param szRange range for the width and height of the box.
 * \param alpha must be a nonnegative number, the bigger alpha, the larger
 *    the returned rectangle tends to be.
 * \param tolFac tolerance factor for stopping
 * \return The best rectangle that minimizes the desired energy
 */
Window findBox1j1(int **kpAssign, int nKeyPnt, int imH, int imW,
      double **B, int nHistBin, int nPC, SizeRange szRange, double alpha, double tolFac);


/**
 * Find the best rectangular window in an image that maximizes:
 *    \f$ \frac{||Bh||^2}{||h||^2 + \alpha} \f$
 * \param kpAssign a 2-dim array of size nKeyPnt*3 for histogram bin assignments.
 *    kpAssign[i][0], kpAssign[i][1], kpAssign[i][2] are row#, col#, bin#
 *       entries are positive integers.
 * \param nKeyPnt # of keypoints.
 * \param imH, imW height and width of the image
 * \param B nPC*nHistBin matrix for the PCA ORTHORNORMAL basis
 * \param nHistBin # of histogram bins
 * \param nPC # of principal components
 * \param szRange range for the width and height of the box.
 * \param alpha must be a nonnegative number, the bigger alpha, the larger
 *    the returned rectangle tends to be.
 * \param tolFac tolerance factor for stopping
 * \return The best rectangle that minimizes the desired energy
 */
Window findBox1j2(int **kpAssign, int nKeyPnt, int imH, int imW,
      double **B, int nHistBin, int nPC, SizeRange szRange, double alpha, double tolFac);


#endif /* M_TREENODE1J_H_ */
