/*
 * m_TreeNode2c.h
 *
 * Created on: Oct 16, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#ifndef M_TREENODE2C_H_
#define M_TREENODE2C_H_

#include "m_TreeNode2.h"

/**
 *
 */
class TreeNode2c:public TreeNode2{
public:
   static int ***integralIms1, ***integralIms2, nHistBin;
   static SizeRange szRange1, szRange2;
   static double alpha;

   void cmpBnds();

   TreeNode2c(WindowPair winPair1_, WindowPair winPair2_):
      TreeNode2(winPair1_, winPair2_){};
   TreeNode2c(){};
};


/**
 * Find the best rectangular windows in two images that maximize the
 * normalized cross correlation: \f$ \frac{h_1^T h_2}{||h_1||||h_2|| + \alpha} \f$
 * \param kpAssign1 a 2-dim array of size nKeyPnt1*3 for histogram bin assignments.
 *    kpAssign1[i][0], kpAssign1[i][1], kpAssign1[i][2] are row#, col#, bin#
 *       entries are positive integers.
 * \param nKeyPnt1 # of keypoints for the first image.
 * \param imH1, imW1 height and width of the first image
 * \param szRange1 range for the width and height of the box.
 * \param nHistBin # of histogram bins
 * \param alpha must be a nonnegative number, the bigger alpha, the larger
 *    the returned rectangles tend to be.
 * \param tolFac tolerance factor for stopping
 * \return The best rectangles that maximize the above desired energy
 */
WindowPair findBoxes2c(
      int **kpAssign1, int nKeyPnt1, int imH1, int imW1, SizeRange szRange1,
      int **kpAssign2, int nKeyPnt2, int imH2, int imW2, SizeRange szRange2,
      int nHistBin, double alpha, double tolFac);



#endif /* M_TREENODE2C_H_ */
