/*
 * m_TreeNode3.cpp
 *
 * Created on: May 8, 2010
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#include "m_malloc.h"
#include "m_utils.h"
#include "m_BranchBound.cpp"
#include "m_TreeNode3.h"
#include <sstream>

double
VEvt::getEnergy(NumPair gtEvt, double **integralIms, double *mu){
   NumPair vSeg = NumPair(start, end);
      //truncated gt event
   NumPair trGtSeg = NumPair(gtEvt.l, min(gtEvt.u, time));

   double *hist_ub = vSeg.getPosNegScore(integralIms);
   double energy = hist_ub[0] - hist_ub[1];
   double *trGtHist_ub = trGtSeg.getPosNegScore(integralIms);
   energy -= trGtHist_ub[0] - trGtHist_ub[1];
   energy -= trGtSeg.F1Loss(vSeg);
   energy *= mu[time];
   destroyVector<double>(hist_ub, 2);
   destroyVector<double>(trGtHist_ub,2);
   return energy;
}

void
VEvt::copy(VEvt *newVEvt){
   newVEvt->start = start;
   newVEvt->end = end;
   newVEvt->time = time;
}
string
VEvt::str(){
   ostringstream rslt;
   rslt << "(" << start << ", " << end << ", " << time << ")";
   return rslt.str();
}

bool VEvt::isSame(VEvt other){
   return ((start == other.start) && (end == other.end) && (time == other.time));
}
void
TreeNode3::copy(TreeNode3 *newTrN){
   newTrN->startBnd = startBnd;
   newTrN->endBnd = endBnd;
   newTrN->timeBnd = timeBnd;
   newTrN->ubEvt = ubEvt;
}

void
TreeNode3::split(TreeNode3 *left, TreeNode3 *right){
   int startBndL  = startBnd.u - startBnd.l + 1;
   int endBndL    = endBnd.u   - endBnd.l   + 1;
   int timeBndL   = timeBnd.u  - timeBnd.l  + 1;

   // find the bounds with longest ranges
   int longestBndIdx = 1;
   int longestBndL = startBndL;
   if (longestBndL < endBndL){
      longestBndIdx = 2;
      longestBndL = endBndL;
   }
   if (longestBndL < timeBndL){
      longestBndIdx = 3;
      longestBndL = timeBndL;
   }
   switch(longestBndIdx){
   case 1:
      copy(left); copy(right);
      startBnd.split(&left->startBnd, &right->startBnd);
      break;
   case 2:
      copy(left); copy(right);
      endBnd.split(&left->endBnd, &right->endBnd);
      break;
   case 3:
      copy(left); copy(right);
      timeBnd.split(&left->timeBnd, &right->timeBnd);
      break;
   }
}

VEvt
TreeNode3::getShortestVEvt(){
   if ((startBnd.u <= endBnd.l) && (endBnd.l <= timeBnd.u))
      return VEvt(startBnd.u, endBnd.l, timeBnd.u);
   else return VEvt();
}

VEvt
TreeNode3::getLongestVEvt(){
   if (timeBnd.u >= endBnd.l){
      int u = min(timeBnd.u, endBnd.u);
      if (startBnd.l <= u) return VEvt(startBnd.l, u, timeBnd.u);
      else return VEvt();
   } else return VEvt();
}

NumPair
TreeNode3::getShortestTrGtSeg(){
   if (gtEvt.l <= timeBnd.l) return NumPair(gtEvt.l, min(gtEvt.u, timeBnd.l));
   else return NumPair();
}

NumPair
TreeNode3::getLongestTrGtSeg(){
   if (gtEvt.l <= timeBnd.u) return NumPair(gtEvt.l, min(timeBnd.u, gtEvt.u));
   else return NumPair();
}

string
TreeNode3::str(){
   ostringstream rslt;
   rslt << "[" << startBnd.str() << ", " << endBnd.str() << ", " << timeBnd.str() << "]";
   return rslt.str();
}

void
TreeNode3::cmpBnds(){
   //compute the lower bound
   if (isEmpty()){
      lb = 0;
      ubEvt = VEvt();
      return;
   }

   VEvt minVEvt = getShortestVEvt(); // shortest event
   NumPair minSeg = NumPair(minVEvt.start, minVEvt.end);
   double *minHist = minSeg.getPosNegScore(integralIms);
   VEvt maxVEvt  = getLongestVEvt(); // longest event
   NumPair maxSeg = NumPair(maxVEvt.start, maxVEvt.end);
   double *maxHist = maxSeg.getPosNegScore(integralIms);

   lb = minHist[0] - maxHist[1];
   NumPair minTrGtSeg = getShortestTrGtSeg();
   double *minTrGtHist = minTrGtSeg.getPosNegScore(integralIms);
   NumPair maxTrGtSeg = getLongestTrGtSeg();
   double *maxTrGtHist = maxTrGtSeg.getPosNegScore(integralIms);
   lb -= maxTrGtHist[0] - minTrGtHist[1];

   NumPair minInter = minSeg.intersectNP(minTrGtSeg);
   lb += ((double)2*minInter.getLen())/((double)maxSeg.getLen() +  maxTrGtSeg.getLen())  - 1;
   lb *= mu[timeBnd.u]; // mu is non-decreasing, and lb should be negative

   //compute the upper bound
   ubEvt = getLongestVEvt();
   ub = ubEvt.getEnergy(gtEvt, integralIms, mu);

   destroyVector<double>(minHist, 2);
   destroyVector<double>(maxHist, 2);
   destroyVector<double>(minTrGtHist, 2);
   destroyVector<double>(maxTrGtHist, 2);
}

bool TreeNode3::isEmpty(){
   if (startBnd.isEmpty() || endBnd.isEmpty() || timeBnd.isEmpty()){
      return true;
   } else if ((startBnd.l > endBnd.u) || (startBnd.l > timeBnd.u) || (endBnd.l > timeBnd.u)){
      return true;
   } else{
      return false;
   }
}


double **TreeNode3::integralIms = NULL;
NumPair TreeNode3::gtEvt;
double *TreeNode3::mu = NULL;

VEvt findBox3(double *pixVals, int len, NumPair gtEvt, double *mu, double tolFac, double &energy){
   TreeNode3 rootNode(NumPair(0, gtEvt.u), NumPair(0, gtEvt.u), gtEvt);
   //TreeNode3 rootNode(NumPair(0, len-1), NumPair(0, len-1), NumPair(len-1, len-1));
   double **integralIms = cmpIntegralIms_1D(pixVals, len);
   TreeNode3::integralIms = integralIms;
   TreeNode3::gtEvt = gtEvt;
   TreeNode3::mu = mu;
   TreeNode3 bestNode = m_branch_bound<TreeNode3>(rootNode, tolFac);
   energy = bestNode.ubEvt.getEnergy(gtEvt, integralIms, mu);
   destroyMatrix<double>(integralIms, 2, len);
   return bestNode.ubEvt;
}

VEvt findBox3b(double *pixVals, int len, NumPair gtEvt, double *mu, double tolFac, double &energy){
   TreeNode3 rootNode(NumPair(0, len-1), NumPair(0, len-1), NumPair(len-1, len-1));
   //TreeNode3 rootNode(NumPair(0, len-1), NumPair(0, len-1), NumPair(len-1, len-1));
   double **integralIms = cmpIntegralIms_1D(pixVals, len);
   TreeNode3::integralIms = integralIms;
   TreeNode3::gtEvt = gtEvt;
   TreeNode3::mu = mu;
   TreeNode3 bestNode = m_branch_bound<TreeNode3>(rootNode, tolFac);
   energy = bestNode.ubEvt.getEnergy(gtEvt, integralIms, mu);
   destroyMatrix<double>(integralIms, 2, len);
   return bestNode.ubEvt;
}
