/*
 * m_TreeNode3_tests.cpp
 *
 * Created on: May 8, 2010
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#include <iostream>
#include <cmath>
#include <queue>
#include <string>
#include <sstream>
#include <fstream>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_TreeNode3.h"
#include "m_BranchBound.cpp"
#include "m_tests.h"


VEvt findBox3_slow(double *pixVals, int len, NumPair gtEv, double *mu, double &energy){
   double *WtIntD = new double[len + 1];
   WtIntD[0] = 0;
   for (int i=1; i <=len; i++) WtIntD[i] = WtIntD[i-1] + pixVals[i-1];

   double val, mnVal = numeric_limits<double>::infinity();
   VEvt mvc;
   for (int t = gtEv.l; t <= gtEv.u; t++){
      for (int s=0; s <=t; s++){
         for (int e=s; e <= t; e++){
            val = mu[t]*((WtIntD[e+1] - WtIntD[s]) - (WtIntD[t+1] - WtIntD[gtEv.l]) -
                  NumPair(gtEv.l, t).F1Loss(NumPair(s, e)));
            if (val < mnVal){
               mnVal = val;
               mvc.start   = s;
               mvc.end     = e;
               mvc.time    = t;
            }
         }
      }
   }
   delete [] WtIntD;
   energy = mnVal;
   return mvc;
}


VEvt findBox3b_slow(double *pixVals, int len, NumPair gtEv, double *mu, double &energy){
   double *WtIntD = new double[len + 1];
   WtIntD[0] = 0;
   for (int i=1; i <=len; i++) WtIntD[i] = WtIntD[i-1] + pixVals[i-1];

   double val, mnVal = numeric_limits<double>::infinity();

   double evVal = WtIntD[gtEv.u + 1] - WtIntD[gtEv.l];
   VEvt mvc;
   for (int s=0; s <len; s++){
      for (int e=s; e <len; e++){
         val = (WtIntD[e+1] - WtIntD[s]) - evVal - gtEv.F1Loss(NumPair(s, e));
         if (val < mnVal){
            mnVal = val;
            mvc.start = s;
            mvc.end = e;
            mvc.time = len - 1;
         }
      }
   }
   delete [] WtIntD;
   energy = mnVal;
   return mvc;
}


void test3_2(){
   int nTest = 1000;
   int capVal = 20;
   int n, e, s, nFail = 0, nEPass= 0;
   for (int i=0; i < nTest; i++){
      cout << "---------Test " << i << endl;
      n = rand() % 40 + 30;
      double *pixVals = new double[n];
      for (int j=0; j < n; j++){
         pixVals[j] = (rand() % capVal) - capVal/2 + 0.01;
      }

      s = rand() % n;
      e = rand() % n;
      if (e < s){
         int tmp = e;
         e = s;
         s = tmp;
      }

      NumPair gtEv(s, e);
      double *mu = new double[n];
      for (int j=0; j < s; j++) {
         mu[j] = 1;
         cout << pixVals[j] << " ";
      }
      cout << "[";
      for (int j=s; j <= e; j++) {
         mu[j] = (j-s+1)/(e-s+1);
         cout << pixVals[j] << " ";
      }
      cout << "] ";
      for (int j=e+1; j < n; j++) {
         mu[j] = 1;
         cout << pixVals[j] << " ";
      }
      cout << endl;


      double e1, e2, e3, e4;
      VEvt ev1 = findBox3(pixVals, n, gtEv, mu, 0, e1);
      VEvt ev2 = findBox3_slow(pixVals, n, gtEv, mu, e2);
      cout << "ev1: " << ev1.str() << ", " << "ev2: " << ev2.str() << endl;
      cout << "e1: " << e1 << ", e2: " << e2 << endl;

      VEvt ev3 = findBox3b(pixVals, n, gtEv, mu, 0, e3);
      VEvt ev4 = findBox3b_slow(pixVals, n, gtEv, mu, e4);
      cout << "ev3: " << ev3.str() << ", " << "ev4: " << ev4.str() << endl;
      cout << "e3: " << e3 << ", e4: " << e4 << endl;
      if (((abs(e1 -e2) < 1e-6) &&  ev1.isSame(ev2)) && ((abs(e3 - e4) < 1e-6) && ev3.isSame(ev4))) {
         cout << "ABSOLUTELY PASSED " << endl;
      } else if ((abs(e1 -e2) < 1e-6) && (abs(e3 - e4) < 1e-6)) {
         cout << "ENERGY PASSED" << endl;
         nEPass++;
      } else {
         cout << "=====================================================> FAILED" << endl;
         nFail++;
      }

      delete [] mu;
      delete [] pixVals;
   }
   cout << "Done, nEnergyPass: " << nEPass << ", nFail: " << nFail << ", nTest: "<<  nTest << endl;
 }





void test3_1(){
   int nTest = 1;

   int **imAssigns = buildVector<int*>(nTest);
   int *lens = buildVector<int>(nTest);
   double **weights = buildVector<double*>(nTest);
   int *nHistBins = buildVector<int>(nTest);
   NumPair *gtEvts = new NumPair[nTest];
   VEvt *expEvts = new VEvt[nTest];


   int i = 0;
   nHistBins[i] = 2; lens[i] = 7;
   imAssigns[i] = buildVector<int>(lens[i]);
   imAssigns[i][0] = 1; imAssigns[i][1] = 2; imAssigns[i][2] = 2; imAssigns[i][3] = 2;
   imAssigns[i][4] = 2; imAssigns[i][5] = 1; imAssigns[i][6] = 2;
   weights[i] = buildVector<double>(nHistBins[i]);
   weights[i][0] = 0.5; weights[i][1] = -1;
   gtEvts[i]  = NumPair(4,6);
   expEvts[i] = VEvt(1,6,6);

   for (int k= 0; k < nTest; k++){
      cout << "==============Testing test3_1." << k << endl;
      cout << "signal: " ;
      double *pixVals = buildVector<double>(lens[k]);
      for (int i = 0; i< lens[k]; i++) {
         //printf("%2d ", imAssigns[k][i]);
         pixVals[i] = weights[k][imAssigns[k][i]-1];
         printf("%2g ", pixVals[i]);
      }
      cout << endl;

      double *mu = buildVector<double>(lens[k]);
      for (int i=0; i < gtEvts[k].l; i++) mu[i] = 0;
      for (int i=gtEvts[k].l; i <= gtEvts[k].u; i++)
         mu[i] = (i - gtEvts[k].l + 1)/(gtEvts[k].u - gtEvts[k].l + 1);
      for (int i= gtEvts[k].u+1; i< lens[k]; i++) mu[i] = 1;


      double E;
      VEvt mvEvt = findBox3(pixVals, lens[k], gtEvts[k], mu, 0.0, E);
      double **integralIms = cmpIntegralIms_1D(pixVals, lens[k]);

      double energy_best = mvEvt.getEnergy(gtEvts[k], integralIms, mu);
      double energy_exp  = expEvts[k].getEnergy(gtEvts[k], integralIms, mu);

      cout << "rslt: " << mvEvt.str() << ": " << energy_best << endl;

      if (energy_best < energy_exp){
         cout << "WARNING: expected energy " << energy_exp << " is bigger than returned energy " << energy_best << endl;
      } else if (energy_best > energy_exp){
         cout << "FAILED, expected: " << expEvts[k].str() << "<=============FAILED" << endl;
      } else {
         cout << "PASSED" << endl;
      }

      destroyMatrix<double>(integralIms, 2, lens[k]);
      destroyVector<double>(pixVals, lens[k]);
      destroyVector<double>(mu, lens[k]);
   }

   for (int k=0; k < nTest; k++){
      destroyVector<int>(imAssigns[k], lens[k]);
      destroyVector<double>(weights[k], nHistBins[k]);
   }
   destroyVector<int*>(imAssigns, nTest);
   destroyVector<double*>(weights, nTest);
   destroyVector<int>(lens, nTest);
   destroyVector<int>(nHistBins, nTest);
   delete [] gtEvts;
   delete [] expEvts;
 }


