/*
 * m_multiLabelKSeg_tests.cpp
 *
 * Created on: Nov 30, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#include "m_multiLabelKSeg.h"
#include <iostream>
#include <vector>
#include "m_utils.h"

using namespace std;


class MLKS_TestCase{
public:
   int len, nSeg, nLabel;
   double **A;
   vector<MLKS_Rslt> expAssgn;

   MLKS_TestCase(int len_, int nLabel_, int nSeg_){
      len = len_;
      nLabel = nLabel_;
      nSeg = nSeg_;
      A = buildMatrix<double>(nLabel, len);
   }

   ~MLKS_TestCase(){
      destroyMatrix<double>(A, nLabel, len);
   }
};

class MMC_TestCase{
public:
   int len, nLabel, lmin, lmax;
   double **A;
   vector<MLKS_Rslt> expAssgn;

   MMC_TestCase(int len_, int nLabel_, int lmin_, int lmax_){
      len = len_;
      nLabel = nLabel_;
      lmin = lmin_;
      lmax = lmax_;
      A = buildMatrix<double>(nLabel, len);
   }

   ~MMC_TestCase(){
      destroyMatrix<double>(A, nLabel, len);
   }
};

class KmeansSeg_TestCase{
public:
   int len, d, nLabel, lmin, lmax;
   double **A, **Cs;
   vector<MLKS_Rslt> expAssgn;
   bool shldNrm;

   KmeansSeg_TestCase(int len_, int d_, int nLabel_, int lmin_, int lmax_, bool shldNrm_){
      len = len_;
      d = d_;
      nLabel = nLabel_;
      lmin = lmin_;
      lmax = lmax_;
      shldNrm = shldNrm_;
      A = buildMatrix<double>(len, d);
      Cs = buildMatrix<double>(nLabel, d);
   }

   ~KmeansSeg_TestCase(){
      destroyMatrix<double>(A, len, d);
      destroyMatrix<double>(Cs, nLabel, d);
   }
};

void testMultiLabelKSeg3(){
   int l = 0;
   MMC_TestCase *tc;
   vector<MMC_TestCase*> tests;

   tc = new MMC_TestCase(1, 2, 1, 2);
   l = 0;
   tc->A[l][0] = 0.5;
   l++;
   tc->A[l][0] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,1));
   tests.push_back(tc);


   tc = new MMC_TestCase(2, 2, 1, 2);
   l = 0;
   tc->A[l][0] = 0.5; tc->A[l][1] = 1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -0.5;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,0));
   tests.push_back(tc);

   tc = new MMC_TestCase(2, 2, 2, 2);
   l = 0;
   tc->A[l][0] = 0.5; tc->A[l][1] = 1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -0.5;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,0));
   tests.push_back(tc);

   tc = new MMC_TestCase(2, 2, 2, 2);
   l = 0;
   tc->A[l][0] = 0.5; tc->A[l][1] = 1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 0.7;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,1));
   tests.push_back(tc);

   tc = new MMC_TestCase(2, 2, 1, 2);
   l = 0;
   tc->A[l][0] = 0.9; tc->A[l][1] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.2;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,0));
   tests.push_back(tc);

   tc = new MMC_TestCase(2, 3, 1, 6);
   l = 0;
   tc->A[l][0] = 0.9; tc->A[l][1] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.2;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,0));
   tests.push_back(tc);


   tc = new MMC_TestCase(3, 3, 1, 2);
   l = 0;
   tc->A[l][0] = 0.9; tc->A[l][1] = 0; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.2; tc->A[l][2] = 0.2;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tests.push_back(tc);

   tc = new MMC_TestCase(3, 3, 1, 2);
   l = 0;
   tc->A[l][0] = 0.1; tc->A[l][1] = 0; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.4; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0.5;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,1));
   tc->expAssgn.push_back(MLKS_Rslt(2,2,2));
   tests.push_back(tc);

   tc = new MMC_TestCase(4, 3, 1, 3);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.2; tc->A[l][2] = 0.2; tc->A[l][3] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0.9;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,0));
   tc->expAssgn.push_back(MLKS_Rslt(3,3,2));
   tests.push_back(tc);


   tc = new MMC_TestCase(6, 3, 1, 6);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.2; tc->A[l][2] = 0.2; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0.3; tc->A[l][4] = 0.3; tc->A[l][5] = 0.3;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,0));
   tc->expAssgn.push_back(MLKS_Rslt(3,5,2));
   tests.push_back(tc);

   tc = new MMC_TestCase(6, 3, 1, 6);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.3; tc->A[l][2] = 0.3; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0.3; tc->A[l][4] = 0.3; tc->A[l][5] = 0.3;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,5,2));
   tests.push_back(tc);

   tc = new MMC_TestCase(3, 3, 2, 2);
   l = 0;
   tc->A[l][0] = 0.1; tc->A[l][1] = 0; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.4; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0.5;
   tests.push_back(tc);

   tc = new MMC_TestCase(6, 3, 1, 3);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0.3; tc->A[l][5] = 0.3;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.3; tc->A[l][2] = 0.3; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.1; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,5,0));
   tests.push_back(tc);

   tc = new MMC_TestCase(6, 3, 1, 6);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0.2; tc->A[l][4] = 0.3; tc->A[l][5] = 0.3;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.3; tc->A[l][2] = 0.3; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.1; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   tc->expAssgn.push_back(MLKS_Rslt(0,5,0));
   tests.push_back(tc);

   tc = new MMC_TestCase(6, 3, 1, 6);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0.1; tc->A[l][4] = 0.1; tc->A[l][5] = 0.1;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.3; tc->A[l][2] = 0.3; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 1; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,5,2));
   tests.push_back(tc);

   for (unsigned int i = 0; i < tests.size(); i++){
      cout << endl << "===> testMMCseg # " << i << endl;
      tc = tests[i];
      for (int k=0; k < tc->nLabel; k++){
         for (int j=0; j < tc->len; j++){
            printf("%4.1f ", tc->A[k][j]);
         }
         cout << endl;
      }
      cout << endl;
      double energy;
      vector<MLKS_Rslt> rslt =  multiLabelKseg3(tc->A, tc->nLabel, tc->len, tc->lmin, tc->lmax, &energy);

      for (int j = (int) rslt.size() -1; j >=0; j--){
         cout << "   " << rslt[j].startIdx << ", " << rslt[j].endIdx << ": " << rslt[j].label << endl;
      }
      cout << "   energy " << energy << endl;
      double expEnergy = - cmpMLKS_energy(tc->expAssgn, tc->A);

      if (energy < expEnergy - EPS)
         cout << "WARNING: returned energy is smaller than expected energy: " << expEnergy << endl;
      else if (energy > expEnergy + EPS)
         cout << "FAILED: expected energy: " << expEnergy << endl;
      else
         cout << "PASSED" << endl;
   }

   for (unsigned int i = 0; i < tests.size(); i++){
      cout << endl << "===> testMMCseg # " << i << endl;
      tc = tests[i];
      delete tc;
   }
}


void testKmeansSeg(){
   int l = 0;
   KmeansSeg_TestCase *tc;
   vector<KmeansSeg_TestCase*> tests;

   //KmeansSeg_TestCase(int len_, int d_, int nLabel_, int lmin_, int lmax_, bool shldNrm_);
   tc = new KmeansSeg_TestCase(1, 2, 2, 1, 2, 1);
   l = 0;
   tc->A[l][0] = 0.5; tc->A[l][1] = 1;
   l = 0;
   tc->Cs[l][0] = 0.5; tc->Cs[0][1] = -1;
   l++; tc->Cs[l][0] = 0.5; tc->Cs[0][1] = 0.9;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,1));
   tests.push_back(tc);

   for (unsigned int i = 0; i < tests.size(); i++){
      cout << endl << "===> testKmeansSeg # " << i << endl;
      tc = tests[i];
      printf("D:\n");
      for (int k=0; k < tc->d; k++){
         for (int j=0; j < tc->len; j++){
            printf("%4.1f ", tc->A[j][k]);
         }
         cout << endl;
      }
      cout << endl;

      printf("Cs:\n");
      for (int k=0; k < tc->d; k++){
         for (int j=0; j < tc->nLabel; j++){
            printf("%4.1f ", tc->Cs[j][k]);
         }
         cout << endl;
      }

      cout << endl;
      double energy;

//      vector<MLKS_Rslt> KmeansSeg(double **D, int d, int n,
//            double **Cs, int nClass, int lmin, int lmax, bool shldNorm, double &energy);
      vector<MLKS_Rslt> rslt = KmeansSeg(tc->A, tc->d, tc->len, tc->Cs, tc->nLabel, tc->lmin, tc->lmax, tc->shldNrm, energy);


      for (int j = (int) rslt.size() -1; j >=0; j--){
         cout << "   " << rslt[j].startIdx << ", " << rslt[j].endIdx << ": " << rslt[j].label << endl;
      }
      cout << "   energy " << energy << endl;
//      double expEnergy = cmpMMCS_energy(tc->A, tc->nLabel, tc->len, tc->expAssgn);
//
//      if (energy < expEnergy - EPS)
//         cout << "WARNING: returned energy is smaller than expected energy: " << expEnergy << endl;
//      else if (energy > expEnergy + EPS)
//         cout << "FAILED: expected energy: " << expEnergy << endl;
//      else
//         cout << "PASSED" << endl;
   }

   for (unsigned int i = 0; i < tests.size(); i++){
      cout << endl << "===> testMMCseg # " << i << endl;
      tc = tests[i];
      delete tc;
   }
}




void testMMCSeg(){
   int l = 0;
   MMC_TestCase *tc;
   vector<MMC_TestCase*> tests;

   tc = new MMC_TestCase(1, 2, 1, 2);
   l = 0;
   tc->A[l][0] = 0.5;
   l++;
   tc->A[l][0] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,1));
   tests.push_back(tc);


   tc = new MMC_TestCase(2, 2, 1, 2);
   l = 0;
   tc->A[l][0] = 0.5; tc->A[l][1] = 1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -0.5;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,0));
   tests.push_back(tc);

   tc = new MMC_TestCase(2, 2, 2, 2);
   l = 0;
   tc->A[l][0] = 0.5; tc->A[l][1] = 1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -0.5;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,0));
   tests.push_back(tc);

   tc = new MMC_TestCase(2, 2, 2, 2);
   l = 0;
   tc->A[l][0] = 0.5; tc->A[l][1] = 1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 0.7;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,1));
   tests.push_back(tc);

   tc = new MMC_TestCase(2, 2, 1, 2);
   l = 0;
   tc->A[l][0] = 0.9; tc->A[l][1] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.2;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,0));
   tests.push_back(tc);

   tc = new MMC_TestCase(2, 3, 1, 6);
   l = 0;
   tc->A[l][0] = 0.9; tc->A[l][1] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.2;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,0));
   tests.push_back(tc);


   tc = new MMC_TestCase(3, 3, 1, 2);
   l = 0;
   tc->A[l][0] = 0.9; tc->A[l][1] = 0; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.2; tc->A[l][2] = 0.2;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tests.push_back(tc);

   tc = new MMC_TestCase(3, 3, 1, 2);
   l = 0;
   tc->A[l][0] = 0.1; tc->A[l][1] = 0; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.4; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0.5;
   tc->expAssgn.push_back(MLKS_Rslt(0,1,1));
   tc->expAssgn.push_back(MLKS_Rslt(2,2,2));
   tests.push_back(tc);

   tc = new MMC_TestCase(4, 3, 1, 3);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.2; tc->A[l][2] = 0.2; tc->A[l][3] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0.9;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,0));
   tc->expAssgn.push_back(MLKS_Rslt(3,3,2));
   tests.push_back(tc);


   tc = new MMC_TestCase(6, 3, 1, 6);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.2; tc->A[l][2] = 0.2; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0.3; tc->A[l][4] = 0.3; tc->A[l][5] = 0.3;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,0));
   tc->expAssgn.push_back(MLKS_Rslt(3,5,2));
   tests.push_back(tc);

   tc = new MMC_TestCase(6, 3, 1, 6);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.3; tc->A[l][2] = 0.3; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0.3; tc->A[l][4] = 0.3; tc->A[l][5] = 0.3;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,5,2));
   tests.push_back(tc);

   tc = new MMC_TestCase(3, 3, 2, 2);
   l = 0;
   tc->A[l][0] = 0.1; tc->A[l][1] = 0; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.4; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0.5;
   tests.push_back(tc);

   tc = new MMC_TestCase(6, 3, 1, 3);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0.3; tc->A[l][5] = 0.3;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.3; tc->A[l][2] = 0.3; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.1; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,5,0));
   tests.push_back(tc);

   tc = new MMC_TestCase(6, 3, 1, 6);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0.2; tc->A[l][4] = 0.3; tc->A[l][5] = 0.3;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.3; tc->A[l][2] = 0.3; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.1; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   tc->expAssgn.push_back(MLKS_Rslt(0,5,0));
   tests.push_back(tc);

   tc = new MMC_TestCase(6, 3, 1, 6);
   l = 0;
   tc->A[l][0] = 1; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0.1; tc->A[l][4] = 0.1; tc->A[l][5] = 0.1;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 0.3; tc->A[l][2] = 0.3; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   l++;
   tc->A[l][0] = 0; tc->A[l][1] = 1; tc->A[l][2] = 0; tc->A[l][3] = 0; tc->A[l][4] = 0; tc->A[l][5] = 0;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,5,2));
   tests.push_back(tc);

   for (unsigned int i = 0; i < tests.size(); i++){
      cout << endl << "===> testMMCseg # " << i << endl;
      tc = tests[i];
      for (int k=0; k < tc->nLabel; k++){
         for (int j=0; j < tc->len; j++){
            printf("%4.1f ", tc->A[k][j]);
         }
         cout << endl;
      }
      cout << endl;
      double energy;
      vector<MLKS_Rslt> rslt =  MMCSeg(tc->A, tc->nLabel, tc->len, tc->lmin, tc->lmax, &energy);


      for (int j = (int) rslt.size() -1; j >=0; j--){
         cout << "   " << rslt[j].startIdx << ", " << rslt[j].endIdx << ": " << rslt[j].label << endl;
      }
      cout << "   energy " << energy << endl;
      double expEnergy = cmpMMCS_energy(tc->A, tc->nLabel, tc->len, tc->expAssgn);

      if (energy < expEnergy - EPS)
         cout << "WARNING: returned energy is smaller than expected energy: " << expEnergy << endl;
      else if (energy > expEnergy + EPS)
         cout << "FAILED: expected energy: " << expEnergy << endl;
      else
         cout << "PASSED" << endl;
   }

   for (unsigned int i = 0; i < tests.size(); i++){
      cout << endl << "===> testMMCseg # " << i << endl;
      tc = tests[i];
      delete tc;
   }
}


void testMultiLabelKSeg(){
   int l = 0;
   MLKS_TestCase *tc;
   vector<MLKS_TestCase*> tests;

   tc = new MLKS_TestCase(1, 2, 2);
   l = 0;
   tc->A[l][0] = 0.5;
   l++;
   tc->A[l][0] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tests.push_back(tc);

   tc = new MLKS_TestCase(1, 4, 10);
   l = 0;
   tc->A[l][0] = 0.5;
   l++; tc->A[l][0] = 1;
   l++; tc->A[l][0] = -0.8;
   l++; tc->A[l][0] = 10;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,2));
   tests.push_back(tc);


   tc = new MLKS_TestCase(3, 2, 2);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(4, 2, 4);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,3,0));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 2, 4);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,4,0));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 2, 3);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,4,0));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 3, 3);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 1; tc->A[l][2] = 1; tc->A[l][3] = -2;
   tc->A[l][4] = -2; tc->A[l][5] = 1; tc->A[l][6] = 1; tc->A[l][7] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,4,2));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 3, 3);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 1; tc->A[l][2] = 1; tc->A[l][3] = -2;
   tc->A[l][4] = 1; tc->A[l][5] = 1; tc->A[l][6] = 1; tc->A[l][7] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,4,2));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 3, 4);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 1; tc->A[l][2] = 1; tc->A[l][3] = -2;
   tc->A[l][4] = 1; tc->A[l][5] = 1; tc->A[l][6] = 1; tc->A[l][7] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,3,2));
   tc->expAssgn.push_back(MLKS_Rslt(4,4,0));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 3, 5);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 1; tc->A[l][2] = 1; tc->A[l][3] = -2;
   tc->A[l][4] = 1; tc->A[l][5] = 1; tc->A[l][6] = 1; tc->A[l][7] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,3,2));
   tc->expAssgn.push_back(MLKS_Rslt(4,4,0));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 3, 10);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 1; tc->A[l][2] = 1; tc->A[l][3] = -2;
   tc->A[l][4] = 1; tc->A[l][5] = 1; tc->A[l][6] = 1; tc->A[l][7] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,3,2));
   tc->expAssgn.push_back(MLKS_Rslt(4,4,0));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);


   for (unsigned int i = 0; i < tests.size(); i++){
      cout << endl << "===> testMultiLabelKseg # " << i << endl;
      tc = tests[i];
      for (int k=0; k < tc->nLabel; k++){
         for (int j=0; j < tc->len; j++){
            printf("%4.1f ", tc->A[k][j]);
         }
         cout << endl;
      }
      cout << endl;
      double energy;
      vector<MLKS_Rslt> rslt =  multiLabelKseg(tc->A, tc->nLabel, tc->len, tc->nSeg, &energy);

      for (int j = (int) rslt.size() -1; j >=0; j--){
         cout << "   " << rslt[j].startIdx << ", " << rslt[j].endIdx << ": " << rslt[j].label << endl;
      }
      cout << "   energy " << energy << endl;
      double expEnergy = cmpMLKS_energy(tc->expAssgn, tc->A);

      if (energy < expEnergy - EPS)
         cout << "WARNING: returned energy is smaller than expected energy" << endl;
      else if (energy > expEnergy + EPS)
         cout << "FAILED" << endl;
      else
         cout << "PASSED" << endl;
   }
}


void testMultiLabelKSeg2(){
   int l = 0;
   MLKS_TestCase *tc;
   vector<MLKS_TestCase*> tests;

   tc = new MLKS_TestCase(1, 2, 2);
   l = 0;
   tc->A[l][0] = 0.5;
   l++;
   tc->A[l][0] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tests.push_back(tc);

   tc = new MLKS_TestCase(1, 4, 10);
   l = 0;
   tc->A[l][0] = 0.5;
   l++; tc->A[l][0] = 1;
   l++; tc->A[l][0] = -0.8;
   l++; tc->A[l][0] = 10;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,2));
   tests.push_back(tc);


   tc = new MLKS_TestCase(3, 2, 2);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(4, 2, 4);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,3,0));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 2, 4);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,4,0));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 2, 3);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,4,0));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 3, 3);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 1; tc->A[l][2] = 1; tc->A[l][3] = -2;
   tc->A[l][4] = -2; tc->A[l][5] = 1; tc->A[l][6] = 1; tc->A[l][7] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,4,2));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 3, 3);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 1; tc->A[l][2] = 1; tc->A[l][3] = -2;
   tc->A[l][4] = 1; tc->A[l][5] = 1; tc->A[l][6] = 1; tc->A[l][7] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,4,2));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 3, 4);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 1; tc->A[l][2] = 1; tc->A[l][3] = -2;
   tc->A[l][4] = 1; tc->A[l][5] = 1; tc->A[l][6] = 1; tc->A[l][7] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,3,2));
   tc->expAssgn.push_back(MLKS_Rslt(4,4,0));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 3, 5);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 1; tc->A[l][2] = 1; tc->A[l][3] = -2;
   tc->A[l][4] = 1; tc->A[l][5] = 1; tc->A[l][6] = 1; tc->A[l][7] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,3,2));
   tc->expAssgn.push_back(MLKS_Rslt(4,4,0));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   tc = new MLKS_TestCase(8, 3, 10);
   l = 0;
   tc->A[l][0] = 0; tc->A[l][1] = 0; tc->A[l][2] = 0; tc->A[l][3] = 0;
   tc->A[l][4] = 0; tc->A[l][5] = 0; tc->A[l][6] = 0; tc->A[l][7] = 0;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = -1; tc->A[l][2] = -1; tc->A[l][3] = 1;
   tc->A[l][4] = 1; tc->A[l][5] = -1; tc->A[l][6] = 0; tc->A[l][7] = -1;
   l++;
   tc->A[l][0] = 1; tc->A[l][1] = 1; tc->A[l][2] = 1; tc->A[l][3] = -2;
   tc->A[l][4] = 1; tc->A[l][5] = 1; tc->A[l][6] = 1; tc->A[l][7] = 1;
   tc->expAssgn.push_back(MLKS_Rslt(0,0,0));
   tc->expAssgn.push_back(MLKS_Rslt(1,2,1));
   tc->expAssgn.push_back(MLKS_Rslt(3,3,2));
   tc->expAssgn.push_back(MLKS_Rslt(4,4,0));
   tc->expAssgn.push_back(MLKS_Rslt(5,7,1));
   tests.push_back(tc);

   for (unsigned int i = 0; i < tests.size(); i++){
      cout << endl << "===> testMultiLabelKseg2 # " << i << endl;
      tc = tests[i];
      for (int k=0; k < tc->nLabel; k++){
         for (int j=0; j < tc->len; j++){
            printf("%4.1f ", tc->A[k][j]);
         }
         cout << endl;
      }
      cout << endl;
      double energy;

      bool **segSwchInd = buildMatrix<bool>(tc->nLabel, tc->nLabel);
      for (int u=0; u < tc->nLabel; u++){
         for (int v=0; v < tc->nLabel; v++){
            segSwchInd[u][v] = true;
         }
         segSwchInd[u][u] = false;
      }


      int *rslt =  multiLabelKseg2(tc->A, tc->nLabel, tc->len, tc->nSeg, segSwchInd, &energy);

      for (int j = 0; j < tc->len; j++) cout << rslt[j] << " ";
      cout << endl;

      cout << "   energy " << energy << endl;
      double expEnergy = cmpMLKS_energy(tc->expAssgn, tc->A);

      if (energy < expEnergy - EPS)
         cout << "WARNING: returned energy is smaller than expected energy" << endl;
      else if (energy > expEnergy + EPS)
         cout << "FAILED" << endl;
      else
         cout << "PASSED" << endl;

      destroyMatrix<bool>(segSwchInd, tc->nLabel, tc->nLabel);
      destroyVector<int>(rslt, tc->len);
   }
}
